function [res,Bias,Sigma,beta_bc,D,Omega ] =  kpanelife(Y,X,T,R,h,kernelType,boundaryKern,XTol,maxIter)
% res =  kpanelife(Y,X,T,R,h,kernelType,XTol,maxIter)
%   Estimate a smooth time-varying panel data model as follows,
%   y_it = x_it'beta_t + lambda_it*f_t + u_it
%   Inputs
%       Y = (y_1',...,y_N')': dependent variable, an NT-by-1 vector, where N is
%           number of cross-sections and T is time dimension
%       X: independent variable, an NT-by-p matrix, where p is number of regressors
%       T: time dimension of the panel data
%       R: integer number of latent factors 
%       XTol: error tolerance (optional)
%       maxIter: maximum number of iterations (optional)
%   Outputs: res has the following components
%       beta: a T-by-p matrix of estimated coefficients,
%       C: a NT-by-1 vector of fixed effects, reshape(C,T,N) gives the
%           low-rank matrix of F*L';
%       e: a NT-by-1 vector of residuals, reshape(e,T,N) gives the
%          error matrix of [e_it];
%       L: NT-by-R matrix, Lambda, factor loadings. Assume Lt'Lt/N = I.
%       F: T-by-R matrix, F, factors. Assume F'F is diagonal.
%       flag: 1 indicates convergence, 0 fails to converge
%

[n,p] = size(X);
N = n/T;

if nargin < 9 || isempty(maxIter)
    maxIter = 2000*p;
end
if nargin < 8 || isempty(XTol)
    XTol = 1e-4; 
end
if nargin < 7 || isempty(boundaryKern)
    boundaryKern=1;
end
if nargin < 6 || isempty(kernelType)
    kernelType=1;
end
if nargin < 5 || isempty(h)
    h=2.35/sqrt(12)*T^(-1/5)*N^(-1/10);
end
if nargin < 4 || isempty(R)
    RRtemp = nkfactors(Y,X,N,(1:3)',h,kernelType,boundaryKern,XTol,maxIter,2)
    % R=min(nkfactors(Y,X,N,(1:3)',h,kernelType,boundaryKern,XTol,maxIter))
    R= RRtemp(1); 
end

[beta,Fr,L,flag] = kpanelife_(Y,X,T,R,(1:T)',h,kernelType,boundaryKern,XTol,maxIter);
if isempty(beta)
   disp('bandwidth too small');
   res = [];
   Bias = [];
   Sigma = [];
   beta_bc = [];
   return;
end
C = zeros(T,N);
e = zeros(T,N);
F = zeros(T,R);
for t=1:T
    Xt= X(t:T:n,:); % N-by-p
    Yt = Y(t:T:n,:); % N-by-1
    Lt = reshape(L(t,:),N,R); % N-by-R
    f = (Lt'*Lt)\(Lt'*(Yt-Xt*beta(t,:)')); % R by 1
    F(t,:) = f';
    C(t,:) = f'*Lt'; % 1 by N
    e(t,:) = (Yt-Xt*beta(t,:)')'-C(t,:);
end
C = reshape(C,N*T,1);
e = reshape(e,N*T,1);
L = reshape(L,N*T,R);


res.beta=beta;
res.C = C;
res.e = e;
res.F = F;
res.L = L;
res.flag = flag;
res.R = R;
res.ssr = e'*e;
res.Fr = Fr; 
if nargout>1
    Sigma = zeros(T,p*p);
    D = zeros(T, p*p); 
    Omega = zeros(T, p*p);
    Bias = zeros(T,p);
    beta_bc = zeros(T,p);
    for t=1:T
        [bbc,B,S, Dt, Omegat] = biasterm(t,res,X,T,R,h,kernelType);
        Bias(t,:) = B';
        Sigma(t,:) = reshape(S,1,p*p);
        D(t,:) = reshape(Dt,1, p*p); 
        beta_bc(t,:) = bbc';
        Omega(t,:) = reshape(Omegat,1, p*p); 
    end
end
