function rr = ols(y, X, noconst, display)
% rr = ols(y, X)
% OLS estimation
% Outputs:
%   rr.b: OLS estimator
%   rr.se: standard error of b
%   rr.R2: R^2
%   rr.R2bar: \bar R^2
%   rr.s2: an estimation of sigma^2
%   rr.ssr: sum of square residual;
%   rr.bvar: variance of b
%   rr.resid: residual
%   rr.fit = X*b: fitted value
%   rr.F F stat for the model
%   rr.DW: Durbin Watson stat
%   rr.bhrvar: heteroscedasticity-robust covariance matrix for b
%   rr.hrse: heteroscedasticity-robust standard error
%   rr.adfstat: ADF statistic
%   rr.adfp: p-value for ADF statistic

% Joe Junhui Qian
% Rice Economics
% June 2006

if nargin < 4 || isempty(display)
    display=0;
end

if nargin<3 || isempty(noconst)
    noconst = 0;
end

[n,m] = size(X);

XXinv = (X'*X)\eye(m); 
b = XXinv*X'*y;
e = y - X*b;
ssr = e'*e;
s2 = ssr/(n - m);
bvar = XXinv*s2;
bhrvar = XXinv * (X'*(X.*repmat(e.^2,1,m))) * XXinv;
se = sqrt(diag(bvar));
hrse = sqrt(diag(bhrvar));
y1 = y-mean(y);
R2 = 1 - e'*e / (y1'*y1);
R2bar = 1-(e'*e/(n-m)) / (y1'*y1/(n-1));
if noconst
    F = (y'*y-ssr)/(ssr/(n-m));
else
    F = ((y1'*y1-ssr)/(m-1))/(ssr/(n-m));
end
DW = sum(diff(e).^2)/sum(e.^2);
t = b./se;
pvalue = 2*(1-tcdf(abs(t),n-m));
%if n>=10
%    [h,adfp,adfstat] = adftest(e);
%end
rr.b = b;
rr.se = se;
rr.pvalue = pvalue;
rr.R2 = R2;
rr.R2bar = R2bar;
rr.bvar = bvar;
rr.s2 = s2;
rr.resid = e;
rr.fit = X*b;
rr.ssr = ssr;
rr.F = F;
rr.DW = DW;
rr.bhrvar = bhrvar;
rr.hrse = hrse;
%if n>=10
%    rr.adfstat = adfstat;
%    rr.adfp = adfp;
%end

if nargout < 1 || display
    disp('     Coef Est    S.E.    H.R.S.E.   p-value')
    disp([b se hrse pvalue])
    disp(['N= ' num2str(n) ' SSR= ' num2str(ssr) ' R2= ' num2str(R2) ...
        ' F= ' num2str(F) ' DW = ' num2str(DW)]); %...
%        ' ADF stat = ' num2str(adfstat) ' ADF pvalue = ' num2str(adfp)]);
end

