function [beta,C,F,L,flag,e,Sigma,bc] = panelife(Y,X,T,R,XTol,maxIter)
% Estimate panel data model with interative fixed effects (Bai,2009) 
% [beta,flag,ssr] = panelife(Y,X,T,R,XTol,maxIter)
%   Estimate the following model,
%   y_it = x_it'beta + lambda_i*f_t + u_it
%   Inputs
%       Y = (y_1',...,y_N')': dependent variable, an NT-by-1 vector, where N is
%           number of cross-sections and T is time dimension
%       X: independent variable, an NT-by-p matrix, where p is number of regressors
%       T: time dimension of the panel data
%       R: integer number of latent factors 
%       XTol: error tolerance (optional)
%       maxIter: maximum number of iterations (optional)
%   Outputs:
%       beta: a p-by-1 vector of estimated coefficients,
%       C: a NT-by-1 vector of fixed effects, reshape(C,T,N) gives the
%           low-rank matrix of F*L';
%       F: T-by-R matrix of factors.
%       L: N-by-R matrix factor loadings
%       flag: 1 indicates convergence, 0 fails to converge
%

[n,p] = size(X);

if nargin < 5 || isempty(XTol)
    XTol = 1e-4; 
end
if nargin < 6 || isempty(maxIter)
    maxIter = 2000*p;
end

N = n/T;
beta0 = (X'*X)\X'*Y;
if isnan(sum(beta0)) || isinf(sum(beta0))
    beta0=zeros(p,1);
end
err = 1;
i = 0;
while err > XTol && i < maxIter
    [F,~] = o2(Y,X,T,beta0,R);
    beta1 = o1(Y,X,F);
    err = norm(beta1-beta0);
    beta0 = beta1;
    i = i+1;
%    disp(err);
end
if i<=maxIter
    flag = 1;
else
    flag = 0;
end
beta = beta1;
F = o2(Y,X,T,beta,R); % T by R
e1 = Y-X*beta;
L = (F'*reshape(e1,T,N)/T)';
C = reshape(F*L',N*T,1);
e = Y-X*beta-C;

if nargout > 6
    Pf = F*((F'*F)\F'); % T by T
    Mf = eye(T)-Pf; % 
    A = L*((L'*L)\L')*N;
    Ml = eye(N)-A/N;% N by N

    Z = zeros(n,p);
    for i=1:p
        Xp = reshape(X(:,i),T,N);
        Z(:,i) = reshape(Mf*Xp*Ml,n,1);    
    end
    D = Z'*Z/n;
    
 
    Ze = Z.*repmat(e,1,p);
    D0 = Ze'*Ze/n;
    
    Dinv = D\eye(p);
    Sigma = Dinv*D0*Dinv/(n-(N+T)*R-p);
end

if nargout > 7

    B1 = zeros(p,1);
    B2 = zeros(p,1);
 
    M = 5*T^(1/5);
    for i=1:N
        eps = e((i-1)*T+1:i*T);
        Xi = X((i-1)*T+1:i*T,:);
        for t=1:T-1
            for s=t+1:T
                B1 = B1 - mytrunc(s,t,M)*Pf(t,s)*eps(t)*Xi(s,:)';
            end
        end
    end
    B1 = B1/N;
    
    A1=zeros(N*N,p);
    LF = (L'*L)*(F'*F);
    for pp=1:p
        A1(:,pp) = reshape(L*(LF\F')*reshape(X(:,pp),T,N)*Ml,N*N,1);
    end
    A2=zeros(T*T,p);
    for pp=1:p
        A2(:,pp) = reshape(F*(LF\L')*reshape(X(:,pp),T,N)'*Mf,T*T,1);
    end
    for i=1:N
        for t=1:T
            ee = e((i-1)*T+t)^2;
            B2 = B2 - ee*A1((i-1)*N+i,:)';
        end
    end
    B = Dinv*(B1/T+B2/N);
    bc = beta - B;
end


function k = mytrunc(s,r,M)
tau = (s-r)/M;
if tau>=0 && tau <=1
    k = 1;
else
    k = 0;
end
