function [est,C,F,L,flag,ssr,Sigma] = tvpanelife(Y,X,T,r,Z,regime,XTol,maxIter)
% Given structural breaks, this program estimates parameters in each
% regime. 
% [est,C,F,L,flag,ssr,Sigma] = tvpanelife(Y,X,T,r,Z,regime,XTol,maxIter)
%   Estimate the following model,
%   y_it = x_it'beta_t + z_it'eta + lambda_i*f_t + u_it
%   Inputs
%   Y = (y_1',...,y_N')': dependent variable, an NT-by-1 vector, where N is
%       number of cross-sections and T is time dimension
%   X: independent variable, an NT-by-p matrix, where p is number of regressors
%   T: time dimension of the panel data
%   r: integer number of latent factors 
%   Z: independent variable, an NT-by-q matrix, where q is number of regressors
%   regime: a vector in the form regime = (1,T1,...,Tm,T+1)', where
%       T1,...,Tm are break dates
%   XTol: error tolerance (optional)
%   maxIter: maximum number of iterations (optional)
%   Outputs:
%   est: an (m+1)p+q-by-1 vector of estimated coefficients,
%       reshape(est(1:p(m+1)),p,m+1)' gives alpha, each row of alpha 
%       corresponds to a regime, est(p(m+1)+1:p(m+1)+q) gives eta.
%   C: a NT-by-1 vector of fixed effects, reshape(C,T,N) gives the
%           low-rank matrix of F*L';
%   L: N-by-r matrix, Lambda, factor loadings. Assume L'L/N = I.
%   F: T-by-r matrix, F, factors. Assume F'F is diagonal.
%   flag: 1 indicates convergence, 0 fails to converge
%   ssr: estimated Q_NT in the paper
%   Sigma: the estimated covariance matrix for alpha,
%       [(m+1)p+q]-by-[(m+1)p+q] matrix.
%

[n,p] = size(X);

if nargin < 5 
    Z = []; 
end
if nargin < 6 || isempty(regime)
    regime = [1;T+1]; 
end
if nargin < 7 || isempty(XTol)
    XTol = 1e-4; 
end
if nargin < 8 || isempty(maxIter)
    maxIter = 2000*p;
end

N = n/T;
q = size(Z,2);
m=length(regime)-2;
G = [X Z];
est0 = (G'*G)\G'*Y;
est0 = [repmat(est0(1:p),m+1,1);est0(p+1:p+q)];
 
err = 1;
i = 0;
while err > XTol && i < maxIter
    L = oracle2(Y,X,Z,regime,est0,r);
    est1 = oracle1(Y,X,Z,regime,L);
    err = norm(est1-est0);
    est0 = est1;
    i = i+1;
%    disp([i err]);
end
if i<=maxIter
%    disp('postppc fails to converge, you may increase maxIter.');
    flag = 1;
else
    flag = 0;
end
est = est1;
L = oracle2(Y,X,Z,regime,est,r);%oracle2(Y,X,regime,alpha,r);
M = eye(N)-L*pinv(L'*L)*L';
alpha = reshape(est(1:(m+1)*p),p,m+1)';
eta = est(p*(m+1)+1:p*(m+1)+q);

ssr = 0;
em = zeros(N,T);
for j=1:m+1
    for t=regime(j):regime(j+1)-1
        if q>0
            e = Y(t:T:n) - X(t:T:n,:)*alpha(j,:)'-Z(t:T:n,:)*eta;
        else
            e = Y(t:T:n) - X(t:T:n,:)*alpha(j,:)';
        end
        ssr = ssr + e'*M*e;
        em(:,t) = e;
    end
end
ssr = ssr/n;
F = em'*L/N;
C = reshape(F*L',N*T,1);

if nargout > 5
    Sigma = covmat1(X,Z,N,regime,L,em);
end

function S = covmat(X,Z,N,regime,L,em)

[n,p]=size(X);
q = size(Z,2);
T = n/N;
m = length(regime)-2;

    Sig1 = zeros((m+1)*p+q,(m+1)*p+q);
    M = eye(N)-L*pinv(L'*L)*L';
    F = (L'*em/N)'; % T-by-r
    Chi = F * pinv(F'*F) * F'; % T-by-T
    ems = em*Chi; % N-by-T
    e = em-ems; % N-by-T

    w = zeros(T,T);
    for t=1:T
        for s=1:T
            w(t,s) = e(:,t)'*e(:,s)/N;
        end
    end
    
    for k=1:m+1
        for j=1:m+1
            pp = zeros(p,p);
            for t=regime(k):regime(k+1)-1
                Xt=X(t:T:n,:);
                for s=regime(j):regime(j+1)-1
                    Xs=X(s:T:n,:);
                    pp = pp + w(t,s)*Xt'*M*Xs;
                end
            end
            Sig1((k-1)*p+1:k*p,(j-1)*p+1:j*p)=pp/(N*sqrt((regime(k+1)-regime(k))*(regime(j+1)-regime(j))));
        end
    end
    
    if q>0 % if Z exists
    for j=1:m+1
        pq = zeros(p,q);
        for t=regime(j):regime(j+1)-1
            Xt=X(t:T:n,:);
            for s=1:T
                Zs=Z(s:T:n,:);
                pq = pq + w(t,s)*Xt'*M*Zs;
            end
        end
        Sig1((j-1)*p+1:j*p,(m+1)*p+1:(m+1)*p+q) = pq/(N*sqrt(T*(regime(j+1)-regime(j))));
        Sig1((m+1)*p+1:(m+1)*p+q,(j-1)*p+1:j*p) = pq'/(N*sqrt(T*(regime(j+1)-regime(j))));
    end
    
    qq = zeros(q,q);
    for t=1:T
        Zt=Z(t:T:n,:);
        for s=1:T
            Zs=Z(s:T:n,:);
            qq = qq+w(t,s)*Zt'*M*Zs;
        end
    end
    Sig1((m+1)*p+1:(m+1)*p+q,(m+1)*p+1:(m+1)*p+q)=qq/n;
    end % end
    
    Phi = zeros((m+1)*p+q,(m+1)*p+q);
    for j = 1:m+1
        pp = zeros(p,p);
        for t=regime(j):regime(j+1)-1
            pp = pp+X(t:T:n,:)'*M*X(t:T:n,:);
        end
        Phi((j-1)*p+1:j*p,(j-1)*p+1:j*p) = pp/(N*(regime(j+1)-regime(j)));
    end
    
    if q>0
    for j = 1:m+1
        pq = zeros(p,q);
        for t=regime(j):regime(j+1)-1
            pq = pq+X(t:T:n,:)'*M*Z(t:T:n,:);
        end
        Phi((j-1)*p+1:j*p,(m+1)*p+1:(m+1)*p+q) = pq/(N*(regime(j+1)-regime(j)));
        Phi((m+1)*p+1:(m+1)*p+q,(j-1)*p+1:j*p) = pq'/(N*T);
    end
    qq=zeros(q,q);
    for t=1:T
        qq = qq+Z(t:T:n,:)'*M*Z(t:T:n,:);
    end
    Phi((m+1)*p+1:(m+1)*p+q,(m+1)*p+1:(m+1)*p+q) = qq/(N*T);
    end
    
    Phis = zeros((m+1)*p+q,(m+1)*p+q);
    for k = 1:m+1
        for j = 1:m+1
            pp = zeros(p,p);
            for t=regime(k):regime(k+1)-1
                for s = regime(j):regime(j+1)-1
                    pp = pp+Chi(t,s)*X(t:T:n,:)'*M*X(s:T:n,:);
                end
            end
            Phis((k-1)*p+1:k*p,(j-1)*p+1:j*p) = pp/(n*(regime(k+1)-regime(k)));
        end
    end
    
    if q>0
    for j=1:m+1
        pq=zeros(p,q);
        for t=regime(j):regime(j+1)-1
            for s=1:T
                pq = pq + Chi(t,s)*X(t:T:n,:)'*M*Z(s:T:n,:);
            end
        end
        Phis((j-1)*p+1:j*p,(m+1)*p+1:(m+1)*p+q) = pq/(n*(regime(j+1)-regime(j)));
        Phis((m+1)*p+1:(m+1)*p+q,(j-1)*p+1:j*p) = pq'/(n*T);
    end
    qq = zeros(q,q);
    for t=1:T
        for s=1:T
            qq = qq+Chi(t,s)*Z(t:T:n,:)'*M*Z(t:T:n,:);
        end
    end
    Phis((m+1)*p+1:(m+1)*p+q,(m+1)*p+1:(m+1)*p+q) = qq/(n*T);
    end
    
    tau = diff(regime);
    D = kron(diag(1./sqrt(N*tau)),eye(p));
    if q>0
        D = [D zeros(p*(m+1),q);zeros(q,p*(m+1)) kron(1./sqrt(N*T),eye(q))];
    end
    Sig0 = Phi - Phis;
    A = pinv(Sig0);
    S = D*A*Sig1*A'*D;
    
function S = covmat1(X,Z,N,regime,L,em)

[n,p]=size(X);
q = size(Z,2);
T = n/N;
m = length(regime)-2;

    Sig1 = zeros((m+1)*p+q,(m+1)*p+q);
    M = eye(N)-L*pinv(L'*L)*L';
    F = (L'*em/N)'; % T-by-r
    Chi = F * pinv(F'*F) * F'; % T-by-T
    ems = em*Chi; % N-by-T
    e = em-ems; % N-by-T

    for k=1:m+1
        for j=1:m+1
            pp = zeros(p,p);
            for t=regime(k):regime(k+1)-1
                Xt=X(t:T:n,:);
                for s=regime(j):regime(j+1)-1
                    Xs=X(s:T:n,:);
                    pp = pp + (Xt.*repmat(e(:,t),1,p))'*M*(Xs.*repmat(e(:,s),1,p));
                end
            end
            Sig1((k-1)*p+1:k*p,(j-1)*p+1:j*p)=pp/(N*sqrt((regime(k+1)-regime(k))*(regime(j+1)-regime(j))));
        end
    end
    
    if q>0 % if Z exists
    for j=1:m+1
        pq = zeros(p,q);
        for t=regime(j):regime(j+1)-1
            Xt=X(t:T:n,:);
            for s=1:T
                Zs=Z(s:T:n,:);
                pq = pq + (Xt.*repmat(e(:,t),1,p))'*M*(Zs.*repmat(e(:,s),1,q));
            end
        end
        Sig1((j-1)*p+1:j*p,(m+1)*p+1:(m+1)*p+q) = pq/(N*sqrt(T*(regime(j+1)-regime(j))));
        Sig1((m+1)*p+1:(m+1)*p+q,(j-1)*p+1:j*p) = pq'/(N*sqrt(T*(regime(j+1)-regime(j))));
    end
    
    qq = zeros(q,q);
    for t=1:T
        Zt=Z(t:T:n,:);
        for s=1:T
            Zs=Z(s:T:n,:);
            qq = qq+(Zt.*repmat(e(:,t),1,q))'*M*(Zs.*repmat(e(:,s),1,q));
        end
    end
    Sig1((m+1)*p+1:(m+1)*p+q,(m+1)*p+1:(m+1)*p+q)=qq/n;
    end % end
    
    Phi = zeros((m+1)*p+q,(m+1)*p+q);
    for j = 1:m+1
        pp = zeros(p,p);
        for t=regime(j):regime(j+1)-1
            pp = pp+X(t:T:n,:)'*M*X(t:T:n,:);
        end
        Phi((j-1)*p+1:j*p,(j-1)*p+1:j*p) = pp/(N*(regime(j+1)-regime(j)));
    end
    
    if q>0
    for j = 1:m+1
        pq = zeros(p,q);
        for t=regime(j):regime(j+1)-1
            pq = pq+X(t:T:n,:)'*M*Z(t:T:n,:);
        end
        Phi((j-1)*p+1:j*p,(m+1)*p+1:(m+1)*p+q) = pq/(N*(regime(j+1)-regime(j)));
        Phi((m+1)*p+1:(m+1)*p+q,(j-1)*p+1:j*p) = pq'/(N*T);
    end
    qq=zeros(q,q);
    for t=1:T
        qq = qq+Z(t:T:n,:)'*M*Z(t:T:n,:);
    end
    Phi((m+1)*p+1:(m+1)*p+q,(m+1)*p+1:(m+1)*p+q) = qq/(N*T);
    end
    
    Phis = zeros((m+1)*p+q,(m+1)*p+q);
    for k = 1:m+1
        for j = 1:m+1
            pp = zeros(p,p);
            for t=regime(k):regime(k+1)-1
                for s = regime(j):regime(j+1)-1
                    pp = pp+Chi(t,s)*X(t:T:n,:)'*M*X(s:T:n,:);
                end
            end
            Phis((k-1)*p+1:k*p,(j-1)*p+1:j*p) = pp/(n*(regime(k+1)-regime(k)));
        end
    end
    
    if q>0
    for j=1:m+1
        pq=zeros(p,q);
        for t=regime(j):regime(j+1)-1
            for s=1:T
                pq = pq + Chi(t,s)*X(t:T:n,:)'*M*Z(s:T:n,:);
            end
        end
        Phis((j-1)*p+1:j*p,(m+1)*p+1:(m+1)*p+q) = pq/(n*(regime(j+1)-regime(j)));
        Phis((m+1)*p+1:(m+1)*p+q,(j-1)*p+1:j*p) = pq'/(n*T);
    end
    qq = zeros(q,q);
    for t=1:T
        for s=1:T
            qq = qq+Chi(t,s)*Z(t:T:n,:)'*M*Z(t:T:n,:);
        end
    end
    Phis((m+1)*p+1:(m+1)*p+q,(m+1)*p+1:(m+1)*p+q) = qq/(n*T);
    end
    
    tau = diff(regime);
    D = kron(diag(1./sqrt(N*tau)),eye(p));
    if q>0
        D = [D zeros(p*(m+1),q);zeros(q,p*(m+1)) kron(1./sqrt(N*T),eye(q))];
    end
    Sig0 = Phi - Phis;
    A = pinv(Sig0);
    S = D*A*Sig1*A'*D;

